@@ -7,11 +7,14 @@
 #'
 #' @inheritParams install_github
 #' @param repo Repository address in the format
-#'   `username/repo[/subdir][@@ref]`.
+#'   `username/repo[@@ref]`.
 #' @param host GitLab API host to use. Override with your GitLab enterprise
-#'   hostname, for example, `"gitlab.hostname.com"`.
+#'   hostname, for example, `"<PROTOCOL://>gitlab.hostname.com"`.
+#'   The PROTOCOL is required by packrat during RStudio Connect deployment. While
+#'   \link{install_gitlab} may work without, omitting it generally
+#'   leads to package restoration errors.
 #' @param auth_token To install from a private repo, generate a personal access
-#'   token (PAT) in \url{https://gitlab.com/profile/personal_access_tokens} and
+#'   token (PAT) in \url{https://docs.gitlab.com/ee/user/profile/personal_access_tokens.html} and
 #'   supply to this argument. This is safer than using a password because you
 #'   can easily delete a PAT without affecting any others. Defaults to the
 #'   GITLAB_PAT environment variable.
@@ -23,18 +26,20 @@
 #' install_gitlab("jimhester/covr")
 #' }
 install_gitlab <- function(repo,
-                           auth_token = gitlab_pat(),
+                           subdir = NULL,
+                           auth_token = gitlab_pat(quiet),
                            host = "gitlab.com",
                            dependencies = NA,
                            upgrade = c("default", "ask", "always", "never"),
                            force = FALSE,
                            quiet = FALSE,
                            build = TRUE, build_opts = c("--no-resave-data", "--no-manual", "--no-build-vignettes"),
+                           build_manual = FALSE, build_vignettes = FALSE,
                            repos = getOption("repos"),
                            type = getOption("pkgType"),
                            ...) {
 
-  remotes <- lapply(repo, gitlab_remote, auth_token = auth_token, host = host)
+  remotes <- lapply(repo, gitlab_remote, subdir = subdir, auth_token = auth_token, host = host)
 
   install_remotes(remotes, auth_token = auth_token, host = host,
                   dependencies = dependencies,
@@ -43,22 +48,24 @@ install_gitlab <- function(repo,
                   quiet = quiet,
                   build = build,
                   build_opts = build_opts,
+                  build_manual = build_manual,
+                  build_vignettes = build_vignettes,
                   repos = repos,
                   type = type,
                   ...)
 }
 
-gitlab_remote <- function(repo,
+gitlab_remote <- function(repo, subdir = NULL,
                        auth_token = gitlab_pat(), sha = NULL,
                        host = "gitlab.com", ...) {
 
   meta <- parse_git_repo(repo)
-  meta$ref <- meta$ref %||% "master"
+  meta$ref <- meta$ref %||% "HEAD"
 
   remote("gitlab",
     host = host,
-    repo = meta$repo,
-    subdir = meta$subdir,
+    repo = paste(c(meta$repo, meta$subdir), collapse = "/"),
+    subdir = subdir,
     username = meta$username,
     ref = meta$ref,
     sha = sha,
@@ -70,15 +77,17 @@ gitlab_remote <- function(repo,
 remote_download.gitlab_remote <- function(x, quiet = FALSE) {
   dest <- tempfile(fileext = paste0(".tar.gz"))
 
-  src_root <- build_url(x$host, x$username, x$repo)
-  src <- paste0(src_root, "/repository/archive.tar.gz?ref=", utils::URLencode(x$ref, reserved = TRUE))
+  project_id <- gitlab_project_id(x$username, x$repo, x$ref, x$host, x$auth_token)
+
+  src_root <- build_url(x$host, "api", "v4", "projects", project_id)
+  src <- paste0(src_root, "/repository/archive.tar.gz?sha=", utils::URLencode(x$ref, reserved = TRUE))
 
   if (!quiet) {
     message("Downloading GitLab repo ", x$username, "/", x$repo, "@", x$ref,
             "\nfrom URL ", src)
   }
 
-  download(dest, src, auth_token = x$auth_token, auth_phrase = "private_token=")
+  download(dest, src, headers = c("Private-Token" = x$auth_token))
 }
 
 #' @export
@@ -106,16 +115,27 @@ remote_metadata.gitlab_remote <- function(x, bundle = NULL, source = NULL, sha =
 remote_package_name.gitlab_remote <- function(remote, ...) {
 
   tmp <- tempfile()
-  src <- build_url(
-    remote$host, remote$username, remote$repo, "raw",
-    remote$ref, remote$subdir, "DESCRIPTION")
+
+  src_root <- build_url(
+    remote$host, "api", "v4", "projects",
+    utils::URLencode(paste0(remote$username, "/", remote$repo),
+                     reserved = TRUE),
+    "repository")
+
+  src <- paste0(
+    src_root, "/files/",
+    ifelse(
+      is.null(remote$subdir),
+      "DESCRIPTION",
+      utils::URLencode(paste0(remote$subdir, "/DESCRIPTION"), reserved = TRUE)),
+    "/raw?ref=", utils::URLencode(remote$ref, reserved = TRUE))
 
   dest <- tempfile()
-  res <- download(dest, src, auth_token = remote$auth_token, auth_phrase = "private_token=")
+  res <- download(dest, src, headers = c("Private-Token" = remote$auth_token))
 
   tryCatch(
     read_dcf(dest)$Package,
-    error = function(e) NA_character_)
+    error = function(e) remote$repo)
 }
 
 #' @export
@@ -129,15 +149,15 @@ format.gitlab_remote <- function(x, ...) {
   "GitLab"
 }
 
-gitlab_commit <- function(username, repo, ref = "master",
+gitlab_commit <- function(username, repo, ref = "HEAD",
   host = "gitlab.com", pat = gitlab_pat()) {
 
-  url <- build_url(host, "api", "v4", "projects", utils::URLencode(paste0(username, "/", repo), reserved = TRUE), "repository", "commits", ref)
+  url <- build_url(host, "api", "v4", "projects", utils::URLencode(paste0(username, "/", repo), reserved = TRUE), "repository", "commits", utils::URLencode(ref, reserved = TRUE))
 
   tmp <- tempfile()
-  download(tmp, url, auth_token = pat, auth_phrase = "private_token=")
+  download(tmp, url, headers = c("Private-Token" = pat))
 
-  fromJSONFile(tmp)$id
+  json$parse_file(tmp)$id
 }
 
 #' Retrieve GitLab personal access token.
@@ -157,3 +177,14 @@ gitlab_pat <- function(quiet = TRUE) {
   }
   return(NULL)
 }
+
+gitlab_project_id <- function(username, repo, ref = "HEAD",
+  host = "gitlab.com", pat = gitlab_pat()) {
+
+  url <- build_url(host, "api", "v4", "projects", utils::URLencode(paste0(username, "/", repo), reserved = TRUE), "repository", "commits", utils::URLencode(ref, reserved = TRUE))
+
+  tmp <- tempfile()
+  download(tmp, url, headers = c("Private-Token" = pat))
+
+  json$parse_file(tmp)$project_id
+}
